<?php

namespace domain\Order\Order;

use domain\_\AbstractResolver;
use domain\Distribution\Distribution\Distribution;
use domain\Distribution\ProductDistribution\ProductDistributionRepository;
use domain\Feature\Feature\Feature;
use domain\Feature\Feature\FeatureChecker;
use domain\Product\Accessory\Accessory;
use domain\Product\Product\Product;
use yii\base\ErrorException;

class OrderResolver extends AbstractResolver
{
    protected OrderSolver $orderSolver;
    protected OrderRepository $orderRepository;
    protected ProductDistributionRepository $productDistributionRepository;
    protected FeatureChecker $featureChecker;

    public function loadDependencies(): void
    {
        $this->orderSolver = $this->loadService(OrderSolver::class);
        $this->orderRepository = $this->loadService(OrderRepository::class);
        $this->productDistributionRepository = $this->loadService(ProductDistributionRepository::class);
        $this->featureChecker = $this->loadService(FeatureChecker::class);
    }

    public function getProductQuantityByDistribution(Product $product, Distribution $distribution): float
    {
        $ordersArray = $this->orderRepository->findOrdersByDistribution($distribution);
        return $this->orderSolver->getProductQuantity($product, $ordersArray);
    }

    public function getSmallestQuantityAccessory(Product $product): ?int
    {
        $smallestQuantity = null;

        foreach ($product->getProductAccessories() as $productAccessory) {
            $quantityAccessory = $productAccessory->getAccessory()->getQuantity();
            $smallestQuantity = is_null($smallestQuantity) ? $quantityAccessory
                : min($smallestQuantity, $quantityAccessory);
        }

        return $smallestQuantity;
    }

    public function getProductQuantityMax(Product $product, Distribution $distribution): ?float
    {
        $productDistribution = $this->productDistributionRepository->findOneProductDistribution($distribution, $product);
        // @TODO : gérer via une exception
        if(!$productDistribution) {
            return null;
        }

        $quantityMax = $productDistribution->quantity_max;

        if($this->featureChecker->isEnabled(Feature::ALIAS_PRODUCT_ACCESSORY)) {
            $smallestQuantityAccessory = $this->getSmallestQuantityAccessory($product);
            if (!is_null($smallestQuantityAccessory)) {
                $quantityMax = is_null($quantityMax) ? $smallestQuantityAccessory : min($quantityMax, $smallestQuantityAccessory);
            }
        }

        return $quantityMax;
    }

    public function getQuantityOfAccessoryAvailableInDistribution(Accessory $accessory, Distribution $distribution): int
    {
        $quantityOfAccessoryUsed = 0;

        foreach($accessory->getProductAccessories() as $productAccessory) {
            $quantityOfAccessoryUsed += $this->getProductQuantityByDistribution($productAccessory->getProduct(), $distribution);
        }

        return $accessory->getQuantity() - $quantityOfAccessoryUsed;
    }

    public function getSmallestQuantityAccessoryAvailable(Product $product, Distribution $distribution): ?int
    {
        $smallestQuantity = null;

        if($this->featureChecker->isEnabled(Feature::ALIAS_PRODUCT_ACCESSORY)) {
            foreach ($product->getProductAccessories() as $productAccessory) {
                $quantityAccessoryAvailableInDistribution = $this->getQuantityOfAccessoryAvailableInDistribution(
                    $productAccessory->getAccessory(),
                    $distribution
                );
                $smallestQuantity = is_null($smallestQuantity) ? $quantityAccessoryAvailableInDistribution
                    : min($smallestQuantity, $quantityAccessoryAvailableInDistribution);
            }
        }

        return $smallestQuantity;
    }

    public function getProductQuantityRemaining(Product $product, Distribution $distribution): ?float
    {
        $productDistribution = $this->productDistributionRepository->findOneProductDistribution($distribution, $product);
        if(!$productDistribution) {
            return null;
        }

        $quantityOrder = $this->getProductQuantityByDistribution($product, $distribution);
        $quantityRemaining = is_null($productDistribution->quantity_max) ? null
            : ($productDistribution->quantity_max - $quantityOrder);

        // Limitation par nombre d'accessoires disponibles
        if($this->featureChecker->isEnabled(Feature::ALIAS_PRODUCT_ACCESSORY)) {
            $smallestQuantityAccessoryAvailable = $this->getSmallestQuantityAccessoryAvailable($product, $distribution);
            if (!is_null($smallestQuantityAccessoryAvailable)) {
                $quantityRemaining = is_null($quantityRemaining) ? $smallestQuantityAccessoryAvailable
                    : min($quantityRemaining, $smallestQuantityAccessoryAvailable);
            }
        }

        return $quantityRemaining;
    }
}